
# class_CafInfo.py
#    Top-level class used to store information about CAF jobs and processes
#
# [Mark Neubauer 7/15/03]


# Standard modules
import string

# FBSNG modules
from FBS_API import FBSClient

# My modules
from dcParams import *
from dcUtil import *
from class_jobTimes import jobTimes
from class_processInfo import processInfo


# CafInfo class definition
#
# There are two main data structures that are maintained withing this class:
#
# 1) processDict[segmentID]
#      This is a dictionary of processInfo objects, keyed upon the
#      fully-qualified segment identifier. The processInfo object stores basic
#      information about each segment/process in the system, as defined in
#      class_processInfo.py.
#
# 2) jobTimesObj
#      This is an object of type jobTimes, as defined in class_jobTimes.py. It
#      stores information on process timing (e.g. wait or exec time), with
#      most methods keyed upon JID

class CafInfo:
    """Top-level class used to store information about CAF jobs and processes
    """

    # Contructor
    #

    def __init__(self):

        # Create FBS Client object for API to bmgr
        self.fc = FBSClient()

        # Create process dictionary and jobTimes object
        self.processDict = {}
        self.jobTimesObj = jobTimes()
        self.userList = []

        # Be sure these data structues are clear (should already be the case)
        self.clear()


    # Method definition - clear
    #

    def clear(self):
        """ Clear the process dictionary and jobTimes object
        """

        self.processDict.clear()
        self.jobTimesObj.clear()
        self.userList = []

    # Method definition - refresh
    #
    
    def refresh(self):
        """ Clear internal data structures and fill them with information
            aquired from bmgr poll. Return resulting list of section IDs
        """

        self.clear()
        sectionIDList = self.get()

        return sectionIDList
    

    # Method definition - get
    #
    
    def get(self):
        """ Poll bmgr for all processes on the CAF and add this information
            to the CafInfo data structures
        """
        
        # List to hold section IDs to be returned
        sectionIDList = []


        # Loop over all queues returned by bmgr
        #
        
        for qname in self.fc.getQueueList():

            # Get QInfo object from bmgr
            QInfo = self.fc.getQueue(qname)


            # Loop over all sections in this queue
            #
            
            for sectionID in QInfo.Sections:    # sectionID = JID.sectionName


                # Add information on all processes in this section to the
                # CafInfo data stuctures (add is actually done in the
                # addProcesses() method)
                #
                
                try:
                    # Get SectionInfo object from bmgr
                    SectionInfo = self.fc.getSection(sectionID)

                    # Add processes in SectionInfo object to CafInfo data
                    # structures
                    self.addProcesses(SectionInfo)

                    # Append this section ID to the list
                    sectionIDList.append(sectionID)
                    
                except KeyError:
                    stderrMessage("SectionID %s not found" % sectionID)

        return sectionIDList


    # Method definition - addProcesses
    #
    
    def addProcesses(self, SectionInfo):
        """Given a SectionInfo object from the bmgr, fill CafInfo data
           structures with information about the processes within the section
        """


        # Get section ID, procType, start and submit times
        #
        
        JID, sectionName = (SectionInfo.JobID, SectionInfo.Name)
        sectionID   = JID+"."+sectionName       # sectionID = JID.sectionName
        procType    = SectionInfo.ProcType
        queueName   = SectionInfo.Queue
        startTime   = SectionInfo.StartTime
        submitTime  = SectionInfo.SubTime

        # Extract user name (i.e. kerberos principal name) from section ID
        tmpList = string.split(sectionID, "_")
        dummyJID, userName = string.split(tmpList[0], ".")
        if userName not in self.userList:
            self.userList.append(userName)


        # Store submit time in jobTimes object
        self.jobTimesObj.addSubmitTime(JID, userName, submitTime)


        # Loop over all processes in the section and store information in
        # the CafInfo data structures
        #
        
        numProcessesAdded = 0
        for processNum in SectionInfo.ProcStats.keys():

            # Segment ID is key for process dictionary (processDict)
            # Note: segmentID = JID.sectionName.processNum
            segmentID = sectionID+"."+str(processNum)

            # Get process state (pending, running, exited)
            state = SectionInfo.ProcStats[processNum]


            # Create new processInfo object and store into process dictionary
            #
            
            processInfoObj = processInfo(sectionID, processNum, userName,
                                         queueName, procType, submitTime,
                                         state)
            self.processDict[segmentID] = processInfoObj


            # If process has started, get end time (note: this could be None
            # if still running) and store in jobTimes object
            #
            
            if startTime != None:
                endTime = SectionInfo.getProcess(processNum, 0).EndTime
                self.jobTimesObj.addSegment(segmentID, procType, startTime,
                                            endTime)
            numProcessesAdded += 1
            
        return numProcessesAdded


    # Method definition - updateProcessDict
    #

    def updateProcessDict(self, segmentID, processInfoObj):
        """ Update process dictionary given a segment ID and processInfo
            object
        """

        try:
            self.processDict[segmentID] = processInfoObj
        except KeyError:
            stderrMessage("SegmentID %s not found" % segmentID)


    # Method definition - getNumProcInQueues
    #

    def getNumProcUser(self, state, userList=None):
        """ Return the number of processes of a given state and in a supplied
            process type list. If process type list not supplied, match all
            process types
        """


        # Match all process types if list not supplied
        #
        
        if userList == None:
            allUsers = 1
        else:
            allUsers = 0
            

        # Loop over all processes in the process dictionary and increment
        # counter on matches
        #
        
        numProc = 0
        for segmentID in self.processDict.keys():

            # Get processInfo object from dictionary
            processInfoObj = self.processDict[segmentID]

            # Get process type and state from processInfo object
            userName  = processInfoObj.userName
            procState = processInfoObj.state

            # Increment process counter on state/ptype match
            if (allUsers or userName in userList) and procState == state:
                numProc += 1

        return numProc


    # Method definition - getNumProcInPtypes
    #

    def getNumProcInPtypes(self, state, procTypeList=None):
        """ Return the number of processes of a given state and in a supplied
            process type list. If process type list not supplied, match all
            process types
        """


        # Match all process types if list not supplied
        #
        
        if procTypeList == None:
            allPtypes = 1
        else:
            allPtypes = 0
            

        # Loop over all processes in the process dictionary and increment
        # counter on matches
        #
        
        numProc = 0
        for segmentID in self.processDict.keys():

            # Get processInfo object from dictionary
            processInfoObj = self.processDict[segmentID]

            # Get process type and state from processInfo object
            procType  = processInfoObj.procType
            procState = processInfoObj.state

            # Increment process counter on state/ptype match
            if (allPtypes or procType in procTypeList) and procState == state:
                numProc += 1

        return numProc


    # Method definition - getNumProcNotInPtypes
    #

    def getNumProcNotInPtypes(self, state, procTypeList):
        """ Return the number of processes of a given state and *not* in a
            supplied process type list.
        """


        # Loop over all processes in the process dictionary and increment
        # counter if process type *not* in supplied list
        #

        numProc = 0
        for segmentID in self.processDict.keys():

            # Get processInfo object from dictionary
            processInfoObj = self.processDict[segmentID]

            # Get process type and state from processInfo object
            procType = processInfoObj.procType
            procState    = processInfoObj.state

            # Increment process counter if state match and process type is
            # *not* in supplied list
            if (procType not in procTypeList) and procState == state:
                numProc += 1

        return numProc


    # Method definition - writeNumProcesses
    #

    def writeNumProcesses(self):

        numShortPending = self.getNumProcInPtypes("pending", ["short"])
        numShortRunning = self.getNumProcInPtypes("running", ["short"])

        numMediumPending = self.getNumProcInPtypes("pending", ["medium"])
        numMediumRunning = self.getNumProcInPtypes("running", ["medium"])

        numLongPending = self.getNumProcInPtypes("pending", ["long"])
        numLongRunning = self.getNumProcInPtypes("running", ["long"])

        numGroupsPending = self.getNumProcInPtypes("pending", GROUP_PTYPES)
        numGroupsRunning = self.getNumProcInPtypes("running", GROUP_PTYPES)

        ptypesNotOther = ["short", "medium", "long"]
        for ptype in GROUP_PTYPES:
            ptypesNotOther.append(ptype)
        numOtherPending = self.getNumProcNotInPtypes("pending", ptypesNotOther)
        numOtherRunning = self.getNumProcNotInPtypes("running", ptypesNotOther)

        totalPending = numShortPending + numMediumPending + numLongPending + \
                       + numGroupsPending + numOtherPending

        totalRunning = numShortRunning + numMediumRunning + numLongRunning \
                       + numGroupsRunning + numOtherRunning
        
        stdoutMessage(\
            "Pending (%d): %d %d %d %d %d Running (%d): %d %d %d %d %d" % \
            (totalPending, numShortPending, numMediumPending, numLongPending,
             numGroupsPending, numOtherPending,
             totalRunning, numShortRunning, numMediumRunning, numLongRunning,
             numGroupsRunning, numOtherRunning))

        writeData("pending", [numShortPending, numMediumPending, \
                              numLongPending, numGroupsPending, \
                              numOtherPending])
        writeData("running", [numShortRunning, numMediumRunning, \
                              numLongRunning, numGroupsRunning, \
                              numOtherRunning])


    # Method definition - writeProcessTimeSummary
    #

    def writeProccessTimeSummary(self):

        timeListShort, timeListMedium, timeListLong = ([], [], [])
        timeListGroups, timeListOther = ([], [])
        startListShort, startListMedium, startListLong = ([], [], [])
        startListGroups, startListOther = ([], [])
        execListShort, execListMedium, execListLong = ([], [], [])
        execListGroups, execListOther = ([], [])
        JIDList = self.jobTimesObj.getJIDList()
        for JID in JIDList:

            if self.jobTimesObj.getWaitTime(JID) != None:
                procType = self.jobTimesObj.getProcType(JID)
                waitTime = self.jobTimesObj.getWaitTime(JID)
#                stdoutMessage("waitTime: %s %s %s" % (JID, procType, waitTime))
                if procType == "short":
                    timeListShort.append(self.jobTimesObj.getWaitTime(JID))
                if procType == "medium":
                    timeListMedium.append(self.jobTimesObj.getWaitTime(JID))
                if procType == "long":
                    timeListLong.append(self.jobTimesObj.getWaitTime(JID))
                if procType in GROUP_PTYPES:
                    timeListGroups.append(self.jobTimesObj.getWaitTime(JID))
                else:
                    timeListOther.append(self.jobTimesObj.getWaitTime(JID))

            if self.jobTimesObj.getAvgProcessStartTimeDiff(JID) != None:
                procType = self.jobTimesObj.getProcType(JID)
                startTime = self.jobTimesObj.getAvgProcessStartTimeDiff(JID)
#                stdoutMessage("startTime: %s %s %s" % (JID, procType, startTime))
                if procType == "short":
                    startListShort.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                if procType == "medium":
                    startListMedium.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                if procType == "long":
                    startListLong.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                if procType in GROUP_PTYPES:
                    startListGroups.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                else:
                    startListOther.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))

                    
            if self.jobTimesObj.getAvgProcessExecTime(JID) != None:
                procType = self.jobTimesObj.getProcType(JID)
                execTime = self.jobTimesObj.getAvgProcessExecTime(JID)
#                stdoutMessage("execTime: %s %s %s" % (JID, procType, execTime))
                if procType == "short":
                    execListShort.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                if procType == "medium":
                    execListMedium.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                if procType == "long":
                    execListLong.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                if procType in GROUP_PTYPES:
                    execListGroups.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                else:
                    execListOther.append(self.jobTimesObj.getAvgProcessExecTime(JID))

        waitTimeAvgShort = getAvg(timeListShort)
        waitTimeAvgMedium = getAvg(timeListMedium)
        waitTimeAvgLong = getAvg(timeListLong)
        waitTimeAvgGroups = getAvg(timeListGroups)
        waitTimeAvgOther = getAvg(timeListOther)
        if waitTimeAvgShort != None and waitTimeAvgMedium != None and \
           waitTimeAvgLong  != None and waitTimeAvgGroups != None and \
           waitTimeAvgOther != None:
            stdoutMessage("waitTimeAvg = %.1f %.1f %.1f %.1f %.1f" % (waitTimeAvgShort, waitTimeAvgMedium, waitTimeAvgLong, waitTimeAvgGroups, waitTimeAvgOther))
        writeData("wait_time", [waitTimeAvgShort, waitTimeAvgMedium, \
                                waitTimeAvgLong, waitTimeAvgGroups, \
                                waitTimeAvgOther])


        startTimeAvgShort = getAvg(startListShort)
        startTimeAvgMedium = getAvg(startListMedium)
        startTimeAvgLong = getAvg(startListLong)
        startTimeAvgGroups = getAvg(startListGroups)
        startTimeAvgOther = getAvg(startListOther)
        if startTimeAvgShort != None and startTimeAvgMedium != None and \
           startTimeAvgLong  != None and startTimeAvgGroups != None and \
           startTimeAvgOther != None:
            stdoutMessage("startTimeDiffAvg = %.1f %.1f %.1f %.1f %.1f" % (startTimeAvgShort, startTimeAvgMedium, startTimeAvgLong, startTimeAvgGroups, startTimeAvgOther))
        writeData("start_time_diff", [startTimeAvgShort, startTimeAvgMedium, \
                                startTimeAvgLong, startTimeAvgGroups, \
                                startTimeAvgOther])
        
        execTimeAvgShort = getAvg(execListShort)
        execTimeAvgMedium = getAvg(execListMedium)
        execTimeAvgLong = getAvg(execListLong)
        execTimeAvgGroups = getAvg(execListGroups)
        execTimeAvgOther = getAvg(execListOther)
        if execTimeAvgShort != None and execTimeAvgMedium != None and \
           execTimeAvgLong  != None and execTimeAvgGroups != None and \
           execTimeAvgOther != None:
            stdoutMessage("execTimeAvg = %.1f %.1f %.1f %.1f %.1f" % (execTimeAvgShort, execTimeAvgMedium, execTimeAvgLong, execTimeAvgGroups, execTimeAvgOther))
        writeData("exec_time", [execTimeAvgShort, execTimeAvgMedium, \
                                execTimeAvgLong, execTimeAvgGroups, \
                                execTimeAvgOther])



    def writeUserSummary(self):

        for userName in self.userList:
            numPending    = self.getNumProcUser("pending", [userName])
            numRunning    = self.getNumProcUser("running", [userName])
            waitTime      = self.jobTimesObj.getWaitTimeUser(userName)
            startTimeDiff = self.jobTimesObj.getAvgProcessStartTimeDiffUser(userName)
            execTime      = self.jobTimesObj.getAvgProcessExecTimeUser(userName)
            
            writeDataUser(userName, [numPending, numRunning, waitTime,
                                     startTimeDiff, execTime])
