
# class_MyEventListener.py
#      Interface to passive collection of batch information based upon the FBS
#      Event Listener class
#
# The base class provides a interface for collecting batch information based
# upon passive subscription and listening for state changes. This provides a
# much lighter monitoring of batch information compared to active polling of
# the bmgr through the normal FBS API
#
# [Mark Neubauer 7/15/03]


# FBSNG modules
from FBS_API import FBSEventListener
from FBS_API import FBSClient

# My modules
from dcUtil import *
import shared            #pass module for sharing objects between modules
        

# Define my own event listener class
#

class MyEventListener(FBSEventListener):
    """ Interface to passive collection of batch information based upon the FBS
        Event Listener class
    """
    
    # Contructor
    #

    def __init__(self, sid_list):

        # Call the superclass constructor
        FBSEventListener.__init__(self)

        # Subscribe to all sections
        numSubscribed = self.subscribeSections(sid_list)
        stdoutMessage("%d sections subscribed" % numSubscribed)


    # Override base class methods
    # These methods interface to section state changes. Given that we only
    # care about process-based accounting, we are only really interested in
    # the processStateChanged method
    #
    
    def sectionStateChanged(self, sid, old_state, new_state):
        stdoutMessage("Section %s: %s -> %s" % (sid, old_state, new_state))

    def sectionDeleted(self, sid):
        stdoutMessage("Section %s deleted" % sid)

    def processStateChanged(self, sid, procno, old_state, new_state):
        stdoutMessage("Section %s Process %s: %s -> %s" % \
                      (sid, procno, old_state, new_state))


        # Retrieve the processInfo object for this process
        #
        
        segmentID = sid+"."+str(procno)
        processInfoObj = shared.CafInfoObj.processDict[segmentID]


        # Verify that old state matches what is currently stored
        #
        
        if old_state != processInfoObj.state:
            stderrMessage("process state mismatch! %s %s %s" % \
                          (segmentID, processInfoObj.state, old_state))


        # Update process state stored in processInfo object
        #

        processInfoObj.state = new_state
        shared.CafInfoObj.updateProcessDict(segmentID, processInfoObj)


        # Update process start or end times stored in the jobTimes object
        # If process now running, add this process with current time as start
        #                exiting, update this process with current time as end
        #
        # Note: Using the local time on this machine is a bit of a hack to
        # avoid extra polling on the bmgr of process time information
        #
        # This implies reasonable time synchonization between the bmgr machine
        # and the machine running using this class. Hmmm
        #
        
        if (old_state == "pending" and new_state == "running"):
            procType = processInfoObj.procType
            shared.CafInfoObj.jobTimesObj.addSegment(segmentID, procType,
                                                     time.time())
        elif (old_state == "running" and new_state == "exited"):
            shared.CafInfoObj.jobTimesObj.updateSegmentEndTime(segmentID,
                                                               time.time())

        
    # My additional methods
    #
    
    # Subscribe to a list of sections using the subscribe base method
    def subscribeSections(self, sid_list):

        numSubscribed = 0
        for sid in sid_list:
            try: self.subscribe(sid)
            except KeyError:
                stderrMessage("Section does not exist: %s" % sid)
            else:
                numSubscribed = numSubscribed + 1
        return numSubscribed

    # Unsubscribe sections to a list of sections using the subscribe base
    # method. If list not provide, unsubscribe to all sections
    def unsubscribeSections(self, sid_list = None):

        numUnsubscribed = 0
        if sid_list == None:
            self.unsubscribe()
            return -1
        for sid in sid_list:
            try: self.unsubscribe(sid)
            except KeyError:
                stderrMessage("Section does not exist: %s" % sid)
            else:
                numUnsubscribed = numUnsubscribed + 1
        return numUnsubscribed
