
# class_jobTimes.py
#    Store and retrieve information on process timing
#
# [Mark Neubauer 7/15/03]


# Standard modules
import string
import time

# My modules
from dcUtil import *


# jobTimes class definition
#
# There are two main data structures that are maintained withing this class:
#
# 1) submitTimeDict[JID]
#      This is a dictionary of submit times, keyed upon JID
#
# 2) segmentTimeDict[JID]
#      This is a dictionary of process information for each process in a JID,
#      keyed upon JID. Therefore, each dictionary element is actually a list
#      of lists. The inner list represents information for each process:
#            [ segmentID, procType, startTime, endTime ]
#      The outer list is a list of this process information, one for list for
#      each process

class jobTimes:
    """ Store and retrieve information on process timing
    """


    # Contructor
    #

    def __init__(self):

        self.submitTimeDict = {}
        self.segmentTimeDict = {}
        self.submitTimeDictUser = {}
        self.segmentTimeDictUser = {}
        self.clear()


    # Method definition - clear
    #
    
    def clear(self):
        """ Clear the submit and process time dictionaries
        """
        
        self.submitTimeDict.clear()
        self.segmentTimeDict.clear()
        self.submitTimeDictUser.clear()
        self.segmentTimeDictUser.clear()


    # Method definition - getJIDList
    #

    def getJIDList(self):
        """ Return list of JIDs for currently stored jobs
        """

        return self.submitTimeDict.keys()


    # Method definition - getUserList
    #

    def getUserList(self):
        """ Return list of JIDs for currently stored jobs
        """

        return self.submitTimeDictUser.keys()
    

    # Method definition - addSubmitTime
    #

    def addSubmitTime(self, JID, userName, time):
        """ Add time to the submit time dictionary for given JID
        """
        
        self.submitTimeDict[JID] = time
        self.submitTimeDictUser[userName] = time

    
    # Method definition - addSegment
    #

    def addSegment(self, segmentID, procType, startTime, endTime=None):
        """ Add a process to the segment time dictionary
        """

        # Extract JID from segment ID
        JID, sectionName, procNum = string.split(segmentID, ".")
        userName, sectionNum = string.split(sectionName, "_")

        if self.segmentTimeDict.has_key(JID):

            # Retrieve current list of segment information (which is itself
            # a list!) into a temporary list
            tmpList = self.segmentTimeDict[JID]

            # Append given segment information list to temporary list
            tmpList.append([segmentID, procType, startTime, endTime])

            # Re-assign segment time dictionary entry for this JID to
            # newly-appended list
            self.segmentTimeDict[JID] = tmpList
        else:

            # First process for this JID, so create completely new entry
            self.segmentTimeDict[JID] = [[segmentID, procType, startTime,
                                          endTime]]

        if self.segmentTimeDictUser.has_key(userName):

            # Retrieve current list of segment information (which is itself
            # a list!) into a temporary list
            tmpList = self.segmentTimeDictUser[userName]

            # Append given segment information list to temporary list
            tmpList.append([segmentID, procType, startTime, endTime])

            # Re-assign segment time dictionary entry for this user to
            # newly-appended list
            self.segmentTimeDictUser[userName] = tmpList
        else:

            # First process for this JID, so create completely new entry
            self.segmentTimeDictUser[userName] = [[segmentID, procType,
                                                   startTime, endTime]]

    # Method definition - updateSegmentEndTime
    #

    def updateSegmentEndTime(self, inSegmentID, inEndTime):
        """ Update end time for a process in the segment time dictionary
        """

        # Extract JID from segment ID
        JID, sectionName, procNum = string.split(inSegmentID, ".")

        # Verify that some process for the process' JID has been previously
        # added
        if not self.segmentTimeDict.has_key(JID):
            stderrMessage("No JID=%s in segmentTimeDict" % JID)
            return -1


        # Loop over all processes corresponding to this JID to look for the
        # process matching our segmentID. Record the array index of the match,
        # if found
        #
        
        segmentIndex = -1

        # Get list of process information for this JID
        segmentList = self.segmentTimeDict[JID]
        
        for i in range(len(segmentList)):

            # Get segment ID from list of process information
            segmentID, procType, startTime, endTime = segmentList[i]

            # If the segment of interest is found, record the index and exit
            # loop
            if inSegmentID == segmentID:
                segmentIndex = i
                break

        if segmentIndex == -1:
            stderrMessage("Cannot update segmentID=%s.Not found" % inSegmentID)
            return -1
        else:


            # Found segment of interest, so update segment time dictionary
            # with the end time provided
            #
            
            segmentID, procType, startTime, endTime = segmentList[segmentIndex]
            segmentList[segmentIndex] = segmentID,procType,startTime,inEndTime
            self.segmentTimeDict[JID] = segmentList
            return segmentIndex


    # Method definition - getProcType
    #

    def getProcType(self, JID):
        """ Return process type for a given JID
        """


        # Verify that JID exits in the segment time dictionary
        #
        
        if not self.segmentTimeDict.has_key(JID):
            return None


        # Return the process type for the first process not in the mailer (i.e.
        # end) section
        #
        
        for segmentID,procType,startTime,endTime in self.segmentTimeDict[JID]:
            if string.find(segmentID, "end") == -1:
                return procType
        
    
    # Method definition - getWaitTime
    #

    def getWaitTime(self, JID):
        """ Return time between submitted job and first running process for
            a given JID
        """


        # Verify that JID exits in the both the submit and segment time
        # dictionaries
        #

        if not self.submitTimeDict.has_key(JID):
            return None
        if not self.segmentTimeDict.has_key(JID):
            return None

        # Record submit time
        submitTime = self.submitTimeDict[JID]


        # Create list of process start times
        #
        
        startTimeList = []
        for segmentID,procType,startTime,endTime in self.segmentTimeDict[JID]:
            startTimeList.append(startTime)

        # Sort the list of start times to get the one closest in time to the
        # job submission
        startTimeList.sort()     # increasing order -> more recent time


        # Calculate and return wait time for first process to start
        #
        
        if len(startTimeList) > 0:
            return startTimeList[0] - submitTime
        else:
            return None


    # Method definition - getAvgProcessStartTimeDiff
    #

    def getAvgProcessStartTimeDiff(self, JID):
        """ Return the time difference, averaged over all processes in the
            given JID, for sequential processes to start running
        """


        # Verify that JID exits in the segment time dictionary
        #

        if not self.segmentTimeDict.has_key(JID):
            return None


        # Create list of process start times
        #

        startTimeList = []
        for segmentID,procType,startTime,endTime in self.segmentTimeDict[JID]:
            startTimeList.append(startTime)


        # Sort the list of start times
        startTimeList.sort()     # increasing order -> more recent time


        # Create list of time differences between sequential processes to
        # start running
        #
        
        startTimeDiffList = []
        firstSegment = 1
        for startTime in startTimeList:
            if firstSegment:            # No time difference for first process
                firstSegment = 0
            else:
                startTimeDiff = startTime - startTimeLast
                startTimeDiffList.append(startTimeDiff)
            startTimeLast = startTime


        # Calculate and return average time difference
        #
        
        avgProcessStartTimeDiff = getAvg(startTimeDiffList)
        return avgProcessStartTimeDiff
        

    # Method definition - getAvgProcessExecTime
    #

    def getAvgProcessExecTime(self, JID):
        """ Return average process execution time for given JID
        """


        # Verify that JID exits in the segment time dictionary
        #
        
        if not self.segmentTimeDict.has_key(JID):
            return None
        

        # Create list of process execution times
        #
        
        execTimeList = []
        for segmentID,procType,startTime,endTime in self.segmentTimeDict[JID]:
            if endTime != None:
                execTime = endTime - startTime
                execTimeList.append(execTime)


        # Calculate and return average execution time
        #
        
        avgProcessExecTime = getAvg(execTimeList)
        return avgProcessExecTime

    

    # Method definition - getWaitTimeUser
    #

    def getWaitTimeUser(self, userName):
        """ Return time between submitted job and first running process for
            a given user
        """


        # Verify that user exits in the both the submit and segment time
        # dictionaries
        #

        if not self.submitTimeDictUser.has_key(userName):
            return None
        if not self.segmentTimeDictUser.has_key(userName):
            return None

        # Record submit time
        submitTime = self.submitTimeDictUser[userName]


        # Create list of process start times
        #
        
        startTimeList = []
        for segmentID,procType,startTime,endTime in self.segmentTimeDictUser[userName]:
            startTimeList.append(startTime)

        # Sort the list of start times to get the one closest in time to the
        # job submission
        startTimeList.sort()     # increasing order -> more recent time


        # Calculate and return wait time for first process to start
        #
        
        if len(startTimeList) > 0:
            return startTimeList[0] - submitTime
        else:
            return None


    # Method definition - getAvgProcessStartTimeDiffUser
    #

    def getAvgProcessStartTimeDiffUser(self, userName):
        """ Return the time difference, averaged over all processes in the
            given user, for sequential processes to start running
        """


        # Verify that user exits in the segment time dictionary
        #

        if not self.segmentTimeDictUser.has_key(userName):
            return None


        # Create list of process start times
        #

        startTimeList = []
        for segmentID,procType,startTime,endTime \
                in self.segmentTimeDictUser[userName]:
            startTimeList.append(startTime)


        # Sort the list of start times
        startTimeList.sort()     # increasing order -> more recent time


        # Create list of time differences between sequential processes to
        # start running
        #
        
        startTimeDiffList = []
        firstSegment = 1
        for startTime in startTimeList:
            if firstSegment:            # No time difference for first process
                firstSegment = 0
            else:
                startTimeDiff = startTime - startTimeLast
                startTimeDiffList.append(startTimeDiff)
            startTimeLast = startTime


        # Calculate and return average time difference
        #
        
        avgProcessStartTimeDiff = getAvg(startTimeDiffList)
        return avgProcessStartTimeDiff
        

    # Method definition - getAvgProcessExecTimeUser
    #

    def getAvgProcessExecTimeUser(self, userName):
        """ Return average process execution time for given user
        """


        # Verify that user exits in the segment time dictionary
        #
        
        if not self.segmentTimeDictUser.has_key(userName):
            return None
        

        # Create list of process execution times
        #
        
        execTimeList = []
        for segmentID,procType,startTime,endTime \
                in self.segmentTimeDictUser[userName]:
            if endTime != None:
                execTime = endTime - startTime
                execTimeList.append(execTime)


        # Calculate and return average execution time
        #
        
        avgProcessExecTime = getAvg(execTimeList)
        return avgProcessExecTime

