#!/local/ups/prd/python/v2_1/Linux-2-4/bin/python

# dataCollect.py
#    Top-level script for accumulating data on CAF batch jobs.
#
# The raw data is written to ASCII files which are then read by a separate
# parser daemon and then put into Round Robin Database (RRD) files.
#
# [Mark Neubauer 7/15/03]


# Standard modules
import sys
import time
import os
import string

# FBSNG modules
from class_MyEventListener import MyEventListener

# My modules
from dcParams import *
from dcUtil import *
from class_CafInfo import CafInfo
import shared            #pass module for sharing objects between modules
    

# Main program
#

def main():

    # Create shared CafInfo object
    shared.CafInfoObj = CafInfo()
    

    # Open log file
    #

    fd = os.open(LOG_FILE, os.O_WRONLY | os.O_CREAT | os.O_TRUNC)
    os.dup2(fd, 1)
    os.dup2(fd, 2)
    os.close(fd)


    # Initial polling of section and process information
    #

    stdoutMessage("Initial polling for section list")
    pollStartTime = time.time()
    sectionIDList = shared.CafInfoObj.get()
    timeLastPollCompleted = time.time()
    pollDuration = timeLastPollCompleted - pollStartTime
    stdoutMessage("Polling completed in %d seconds" % pollDuration)


    # Write process accounting information
    #

    shared.CafInfoObj.writeNumProcesses()
    shared.CafInfoObj.writeProccessTimeSummary()
    shared.CafInfoObj.writeUserSummary()
    timeLastDataWriteCompleted = time.time()


    # Initial subscription 
    #

    listener = MyEventListener(sectionIDList)
    numSectionsLast = len(sectionIDList)


    # Loop forever
    #

    while 1:


        # Listen to bmgr information on subscribed sections for
        # POLLING_INTERVAL seconds
        #

        while ((time.time() - timeLastPollCompleted) < POLLING_INTERVAL):


            # Listen for one event or 10 seconds, whichever comes first
            #
            
            nevents = listener.wait(nmax = 1, tmo = 10)


            # Write process information to output files (log and data)
            # regularly (DATA_WRITE_INTERVAL) during listen period (i.e.
            # time between polls for new sections)
            #

            if((time.time()-timeLastDataWriteCompleted) > DATA_WRITE_INTERVAL):

                # Write process accounting (pending + running)
                shared.CafInfoObj.writeNumProcesses()

                # Write job/process time summary (job/process wait times,
                # process execution time)
                shared.CafInfoObj.writeProccessTimeSummary()
                
                # Write user-based summary information
                shared.CafInfoObj.writeUserSummary()

                timeLastDataWriteCompleted = time.time()


        # Refresh section subscription
        #

        # Unsubscribe from all currently subscribed sections. Note: since
        # polling of the bmgr for new sections can take an arbitrary amount
        # of time, need to unsubscribe to all sections to avoid FBS listener
        # socket buffers from filling up. This is very important to avoid
        # hanging up normal bmgr operation!
        listener.unsubscribeSections()


        # Poll bmgr for new sections and refresh CafInfoObj information
        #
        
        stdoutMessage("Polling for new section list")
        pollStartTime = time.time()
        sectionIDList = shared.CafInfoObj.refresh()
        timeLastPollCompleted = time.time()
        pollDuration = timeLastPollCompleted - pollStartTime
        stdoutMessage("Polling completed in %d seconds" % pollDuration)
        

        # Subscribe to the section list resulting from poll
        #
        
        numSubscribed = listener.subscribeSections(sectionIDList)
        netSections = numSubscribed - numSectionsLast
        stdoutMessage(\
            "%5d unsubscribed, %5d subscribed (%d net sections)" % (numSectionsLast, numSubscribed, netSections))
        numSectionsLast = numSubscribed


        # Write process information to output files (log and data)
        #

        # Write process accounting (pending + running)
        shared.CafInfoObj.writeNumProcesses()

        # Write job/process time summary (job/process wait times, process
        # execution time)
        shared.CafInfoObj.writeProccessTimeSummary()

        # Write user-based summary information
        shared.CafInfoObj.writeUserSummary()
        
        timeLastDataWriteCompleted = time.time()
        

if __name__ == "__main__":

    deamonPid = os.fork()
    if deamonPid > 0:
        sys.exit(0)
    else:
        
        # Set FBSNG config file enviroment which describes how to contact
        # the bmgr
        os.environ['FBS_CONFIG'] = FBSNG_CFG_FILE

        # Trying to make this deamon process
        os.setsid()
        # Becomes a session leader of a new session
        # Becomes the process group leader of a new process group
        # Has no controlling terminal

        os.chdir(DC_ROOT_DIR)
        os.umask(0)

        main()
